import enum

import numpy as np


class KernelWarning(enum.Enum):
    INIT = "init"
    MISSINGSOURCE = "missingsource"
    MISSINGKEY = "missingkey"
    PROCESSING = "processing"


class BaseArbiterKernel:
    def __init__(self, config):
        """No need for prefix - hosting device should pass us the relevant subnode"""
        self._config = config
        self._device = None

    @property
    def geometry(self):
        return self._device._geometry

    def warning_context(self, warn_type):
        return self._device.warning_context("frameSelection.kernelState", warn_type)

    @staticmethod
    def extend_device_schema(schema, prefix):
        """Should add configurability to the arbiter (matcher) the kernel will be
        running in.  Prefix will include node name, so will typically be
        "frameSelection.kernels.{class.__name__}". The parent node will not have been
        created already (unlike for correction addons)."""
        pass

    def consider(self, train_id, sources, num_frames):
        """Must return tuple of uint8 mask array of size equal to num_frames
        and base output hash or None."""
        return np.ones(num_frames, dtype=np.uint8), None
