# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

from libc.math cimport isinf, isnan

# TODO: get these automatically from enum definition
cdef unsigned char NONE = 0
cdef unsigned char LUT = 1
cdef unsigned char OFFSET = 2
cdef unsigned char GAIN = 4
cdef unsigned char BPMASK = 8

def correct(
    unsigned short[:, :] raw_data,
    unsigned char[:, :] raw_gain,
    unsigned char flags,
    unsigned short[:, :, :] lut,
    float[:, :, :] offset_map,
    float[:, :, :] gain_map,
    unsigned[:, :, :] badpixel_map,
    float badpixel_fill_value,
    float[:, :] output,
):
    cdef unsigned frame, x
    cdef unsigned short cell, raw_value, looked_up_value
    cdef float res
    cdef unsigned char gain
    for frame in range(raw_data.shape[0]):
        for x in range(raw_data.shape[1]):
            cell = frame % 2
            gain = raw_gain[frame, x]
            if gain == 3:
                gain = 2

            raw_value = raw_data[frame, x]

            if (flags & BPMASK) and badpixel_map[gain, cell, x] != 0:
                res = badpixel_fill_value
            else:
                if (flags & LUT):
                    res = <float>lut[cell, raw_value, x]
                else:
                    res = <float>raw_value

                if (flags & OFFSET):
                    res -= offset_map[gain, cell, x]

                if (flags & GAIN):
                    res /= gain_map[gain, cell, x]

                if isnan(res) or isinf(res):
                    res = badpixel_fill_value

            output[frame, x] = res
