from karabo.bound import (
    IMAGEDATA_ELEMENT,
    INT32_ELEMENT,
    INT64_ELEMENT,
    NDARRAY_ELEMENT,
    NODE_ELEMENT,
    STRING_ELEMENT,
    UINT64_ELEMENT,
    VECTOR_STRING_ELEMENT,
    Schema,
)


def preview_schema(wrap_image_in_imagedata=False):
    res = Schema()
    (
        NODE_ELEMENT(res)
        .key("image")
        .commit(),
    )
    if wrap_image_in_imagedata:
        (
            IMAGEDATA_ELEMENT(res)
            .key("image.data")
            .commit(),

            IMAGEDATA_ELEMENT(res)
            .key("image.mask")
            .commit(),
        )
    else:
        (
            NDARRAY_ELEMENT(res)
            .key("image.data")
            .dtype("FLOAT")
            .commit(),

            NDARRAY_ELEMENT(res)
            .key("image.mask")
            .dtype("UINT8")
            .commit(),

        )
    return res


def xtdf_output_schema(use_shmem_handle=True):
    # TODO: trim output schema / adapt to specific detectors
    # currently: based on snapshot of actual output reusing AGIPD hash
    res = Schema()
    (
        NODE_ELEMENT(res)
        .key("image")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("image.length")
        .dtype("UINT32")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("image.cellId")
        .dtype("UINT16")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("image.pulseId")
        .dtype("UINT64")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("image.status")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("image.trainId")
        .dtype("UINT64")
        .commit(),

        VECTOR_STRING_ELEMENT(res)
        .key("calngShmemPaths")
        .readOnly()
        .commit(),

        NODE_ELEMENT(res)
        .key("metadata")
        .commit(),

        STRING_ELEMENT(res)
        .key("metadata.source")
        .readOnly()
        .commit(),

        NODE_ELEMENT(res)
        .key("metadata.timestamp")
        .commit(),

        INT32_ELEMENT(res)
        .key("metadata.timestamp.tid")
        .readOnly()
        .commit(),

        NODE_ELEMENT(res)
        .key("header")
        .commit(),

        INT32_ELEMENT(res)
        .key("header.minorTrainFormatVersion")
        .readOnly()
        .commit(),

        INT32_ELEMENT(res)
        .key("header.majorTrainFormatVersion")
        .readOnly()
        .commit(),

        INT32_ELEMENT(res)
        .key("header.trainId")
        .readOnly()
        .commit(),

        INT64_ELEMENT(res)
        .key("header.linkId")
        .readOnly()
        .commit(),

        INT64_ELEMENT(res)
        .key("header.dataId")
        .readOnly()
        .commit(),

        INT64_ELEMENT(res)
        .key("header.pulseCount")
        .readOnly()
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("header.reserved")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("header.magicNumberBegin")
        .commit(),

        NODE_ELEMENT(res)
        .key("detector")
        .commit(),

        INT32_ELEMENT(res)
        .key("detector.trainId")
        .readOnly()
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("detector.data")
        .commit(),

        NODE_ELEMENT(res)
        .key("trailer")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("trailer.checksum")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("trailer.magicNumberEnd")
        .commit(),

        INT32_ELEMENT(res)
        .key("trailer.status")
        .readOnly()
        .commit(),

        INT32_ELEMENT(res)
        .key("trailer.trainId")
        .readOnly()
        .commit(),
    )

    if use_shmem_handle:
        (
            STRING_ELEMENT(res)
            .key("image.data")
            .readOnly()
            .commit(),
        )
    else:
        (
            NDARRAY_ELEMENT(res)
            .key("image.data")
            .commit(),
        )
    return res


def jf_output_schema(use_shmem_handle=True):
    res = Schema()
    (
        NODE_ELEMENT(res)
        .key("data")
        .commit(),

        INT32_ELEMENT(res)
        .key("data.trainId")
        .readOnly()
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("data.frameNumber")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("data.gain")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("data.memoryCell")
        .commit(),

        NDARRAY_ELEMENT(res)
        .key("data.timestamp")
        .commit(),

        VECTOR_STRING_ELEMENT(res)
        .key("calngShmemPaths")
        .readOnly()
        .commit(),
    )
    if use_shmem_handle:
        (
            STRING_ELEMENT(res)
            .key("data.adc")
            .readOnly()
            .commit(),
        )
    else:
        (
            NDARRAY_ELEMENT(res)
            .key("data.adc")
            .commit(),
        )
    return res


def pnccd_output_schema(use_shmem_handle=True):
    res = Schema()
    (
        NODE_ELEMENT(res)
        .key("data")
        .commit(),

        UINT64_ELEMENT(res)
        .key("data.trainId")
        .readOnly()
        .commit(),
    )
    if use_shmem_handle:
        (
            STRING_ELEMENT(res)
            .key("data.image")
            .readOnly()
            .commit(),
        )
    else:
        (
            NDARRAY_ELEMENT(res)
            .key("data.image")
            .commit(),
        )
    return res
