import numpy as np
from calng import utils
from calng.corrections.PnccdCorrection import Constants, CorrectionFlags, PnccdCpuRunner


def test_common_mode():
    def slow_common_mode(
        data, bad_pixel_map, noise_map, cm_min_frac, cm_noise_sigma, cm_ss, cm_fs
    ):
        masked = np.ma.masked_array(
            data=data,
            mask=(bad_pixel_map != 0) | (data > noise_map * cm_noise_sigma),
        )

        if cm_fs:
            subset_fs = masked.count(axis=1) >= masked.shape[1] * cm_min_frac
            data[subset_fs] -= np.ma.median(masked[subset_fs], axis=1, keepdims=True)

        if cm_ss:
            subset_ss = masked.count(axis=0) >= masked.shape[0] * cm_min_frac
            data[:, subset_ss] -= np.ma.median(masked[:, subset_ss], axis=0)

    data_shape = (1024, 1024)
    data = (np.random.random(data_shape) * 1000).astype(np.uint16)
    runner = PnccdCpuRunner(1024, 1024, 1, 1)
    for noise_level in (0, 5, 100, 1000):
        noise = (np.random.random(data_shape) * noise_level).astype(np.float32)
        runner.load_constant(Constants.NoiseCCD, noise)
        for bad_pixel_percentage in (0, 1, 20, 80, 100):
            bad_pixels = (
                np.random.random(data_shape) < (bad_pixel_percentage / 100)
            ).astype(np.uint32)
            runner.load_constant(Constants.BadPixelsDarkCCD, bad_pixels)
            runner.load_data(data.copy())
            runner.correct(
                CorrectionFlags.COMMONMODE,
                cm_min_frac=0.25,
                cm_noise_sigma=10,
                cm_row=True,
                cm_col=True,
            )

            slow_version = data.astype(np.float32, copy=True)
            for q_data, q_noise, q_bad_pixels in zip(
                utils.quadrant_views(slow_version),
                utils.quadrant_views(noise),
                utils.quadrant_views(bad_pixels),
            ):
                slow_common_mode(q_data, q_bad_pixels, q_noise, 0.25, 10, True, True)
            assert np.allclose(runner.processed_data, slow_version, equal_nan=True)
